#    most of this task is now taken from the standard application


set frametitle "MIDAS Data Acquisition Experiment Control (TUDA)"

loadsource GroupDefn
loadsource fltohex
loadsource epicsDefn

set ScalerType v560

set Do_Ancillary 0


proc Check-DA {} {
   global OS RDOUThdwre Alive Do_Ancillary
   global CheckDAPeriod

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

#    this check only works if GOING

   if {[:query_Mapped .]} then {

       set z [catch {read-register $RDOUThdwre.Sstate} state]
       if {$z != 0} then {
           if {$CheckDAPeriod > 0} then {after [expr $CheckDAPeriod*1000] Check-DA}
           return
       }
       if {$state != 2} then {return}

       set-footer "Checking Data Acqusition Program"

       set z [catch {read-register $RDOUThdwre.CAlive} Alive]
       if {$z != 0} then {
           if {$CheckDAPeriod > 0} then {after [expr $CheckDAPeriod*1000] Check-DA}
           return
       }

       set z [catch {write-register $RDOUThdwre.CAlive 0}]
       if {$z != 0} then {
           if {$CheckDAPeriod > 0} then {after [expr $CheckDAPeriod*1000] Check-DA}
           return
       }

       if {$Alive == 0} then {
             set-footer "Warning - No data being received or the DA Program has crashed"
       } else {
             set-footer "Data Acquisition running OK"
       }

      click-redisplay

   }
   
   if {$Do_Ancillary >= 0} then {incr Do_Ancillary}
   if {$Do_Ancillary == 6} {

       Do_Scalers
       Do_EPICS
       set-footer "[date] Generated scaler and EPICS data"
       set Do_Ancillary 0
   }

   if {$CheckDAPeriod > 0} then {after [expr $CheckDAPeriod*1000] Check-DA}

}


proc Do_Scalers {} {
    global ScalerType Scaler_Base  EGreport
    
    set EGreport_SAVE $EGreport
    set EGreport -1

     inform $ScalerType display_counters
     set maxcounter [inform $ScalerType inquire maxcounter]

     if {$maxcounter == 0} {return}

     execute-command 10 "Start Scalers Event" "" 2

     for {set i 0} {$i < $maxcounter} {incr i} {
         set scaler [inform $ScalerType inquire data $i]
         set datah [format "%04x" [expr ($scaler >> 16) & 0x0000ffff]]
         set labelh [format "%04x" [expr (($i*2)*256) + $Scaler_Base]]

         set datal [format "%04x" [expr $scaler & 0x0000ffff]]
         set labell [format "%04x" [expr ((($i*2)+1)*256) + $Scaler_Base]]

        execute-command 11 "Define Scalers Event" "$labelh$datah $labell$datal" 2
     }

     execute-command 12 "Complete Scalers Event" "" 2
     
     set EGreport $EGreport_SAVE

}

proc Do_EPICS {} {
    global env EPICS_Base EPICS EGreport

    if {![info exists env(EPICSpresent)]} then {return}
    
#    scan though the EPICS parameter table and generate the command for caget
#    also extract the item base and data mode for use later

    set cmd "exec caget"
    set i 0
    foreach item $EPICS {
        append cmd " [first $item]"
        set id($i) [second $item]
        set mode($i) [third $item]
        incr i
    }
    
#    now execute the caget command 
    
    set z [catch {eval $cmd} m]
    
    if {$z != 0} then {
        set-footer "Warning EPICS channel access failed - $m"
    } else {
    
       set EGreport_SAVE $EGreport
       set EGreport -1
       
#    start generation of the EPICS data event
    
       execute-command 10 "Start EPICS Event" "" 2
       
#    for each line in the response generate a data word
         
       set r [split $m \n]
       set i 0
       foreach item $r {
       
         set vl [second $item]
         
         switch $mode($i) {
         
         0   {
         
#    32 bit floating data

                 set xvl [float2hex $vl]
         
                 set datah  [string range $xvl 2 5]
                 set labelh [format "%04x" [expr ($id($i)*256) + $EPICS_Base]]
         
                 set datal  [string range $xvl 6 9]
                 set labell [format "%04x" [expr (($id($i)+1)*256) + $EPICS_Base]]
         
                 execute-command 11 "Define EPICS Event" "$labelh$datah $labell$datal" 2
              }
              
         1    {
         
#    32 bit integer data

                 set datah  [format "%04x" [expr ($vl >> 16) & 0x0000ffff]]
                 set labelh [format "%04x" [expr ($id($i)*256) + $EPICS_Base]]
         
                 set datal  [format "%04x" [expr $vl & 0x0000ffff]]
                 set labell [format "%04x" [expr (($id($i)+1)*256) + $EPICS_Base]]
         
                 execute-command 11 "Define EPICS Event" "$labelh$datah $labell$datal" 2
              }
              
         2   {
         
#    16 bit integer data

                 set datal  [format "%04x" [expr $vl & 0x0000ffff]]
                 set labell [format "%04x" [expr ($id($i)*256) + $EPICS_Base]]
         
                 execute-command 11 "Define EPICS Event" "$labell$datal" 2
              }
          }
          incr i
       }
       
#    complete the EPICS data event

       execute-command 12 "Complete EPICS Event" "" 2
       
       set EGreport $EGreport_SAVE
    }

}
